/*
 * Created on 6.2.2006
 *
 */
package cz.muni.fi.pb138.demo.transform;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

public class Demo {

    public static Document readAsDOM(File f) {

        // get a DocumentBuilderFactory instance
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        // set the options
        //        factory.setValidating(true);
        factory.setNamespaceAware(true);
        try {
            // get a  DocumentBuilder
            DocumentBuilder builder = factory.newDocumentBuilder();

            // parse the file into a new document
            Document document = builder.parse(f);
            return document;

        } catch (SAXException sxe) {
            // Error generated during parsing
            Exception x = sxe;
            if (sxe.getException() != null)
                x = sxe.getException();
            x.printStackTrace();

        } catch (ParserConfigurationException pce) {
            // Parser with specified options can't be built
            pce.printStackTrace();

        } catch (IOException ioe) {
            // I/O error
            ioe.printStackTrace();
        }
        return null;
    }

    public static Document createDocument() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.newDocument(); // Create from whole cloth

            Element root = (Element) document.createElement("rootElement");
            document.appendChild(root);
            root.appendChild(document.createTextNode("Some"));
            root.appendChild(document.createTextNode(" "));
            root.appendChild(document.createTextNode("text"));

        } catch (ParserConfigurationException pce) {
            // Parser with specified options can't be built
            pce.printStackTrace();
        }
        return document;
    }

    public static void dumpDOM(Document document, OutputStream os)
            throws TransformerException {

        TransformerFactory tf = TransformerFactory.newInstance();

        Transformer transformer = tf.newTransformer();

        DOMSource ds = new DOMSource(document);

        StreamResult sr = new StreamResult(os);

        transformer.transform(ds, sr);
    }

    public static void transformFileToFile(File srcFile, File styleFile,
            File destFile) throws TransformerException {

        TransformerFactory tf = TransformerFactory.newInstance();

        Source styleSource = new StreamSource(styleFile);

        Transformer transformer = tf.newTransformer(styleSource);

        Source source = new StreamSource(srcFile);

        Result result = new StreamResult(destFile);

        transformer.transform(source, result);
    }
    
    public static Templates precompileStyle(File styleFile) throws TransformerConfigurationException {
        
        TransformerFactory tf = TransformerFactory.newInstance();
        
        Source styleSource = new StreamSource(styleFile);
        
        return tf.newTemplates(styleSource);
    }

    public static void transformFileToFileStylePrecompiled(File srcFile, Templates style,
            File destFile) throws TransformerException {

        Transformer transformer = style.newTransformer();

        Source source = new StreamSource(srcFile);

        Result result = new StreamResult(destFile);

        transformer.transform(source, result);
    }

    public static void transformDOM(Document document, File style,
            OutputStream os) throws TransformerException {

        TransformerFactory tf = TransformerFactory.newInstance();

        Source styleSource = new StreamSource(style);

        Transformer transformer = tf.newTransformer(styleSource);

        Source source = new DOMSource(document);

        Result result = new StreamResult(os);

        transformer.transform(source, result);
    }

    public static void createStyleFile() throws FileNotFoundException {

        PrintStream ps = new PrintStream(new FileOutputStream("style.xsl"));

        ps.println("<?xml version='1.0' encoding='UTF-8'?>");
        ps
                .println("<xsl:stylesheet xmlns:xsl='http://www.w3.org/1999/XSL/Transform' version='1.0'>");
        ps.println("<xsl:template match='/rootElement'>");
        ps.println("<xsl:copy>");
        ps.println("New text inserted into root");
        ps.println("...the text continues");
        ps.println("<new_empty_element/>");
        ps.println("<new_nonempty_element attr1='value1'>");
        ps.println("text inside the new element");
        ps.println("</new_nonempty_element>");
        ps.println("</xsl:copy>");
        ps.println("</xsl:template>");
        ps.println("</xsl:stylesheet>");

        ps.close();
    }

    /**
     * @param args
     * @throws TransformerException 
     * @throws FileNotFoundException 
     */
    public static void main(String[] args) throws TransformerException,
            FileNotFoundException {

        createStyleFile();

        System.out.println("--- create an XML file ---");

        Document document = createDocument();

        System.out.println("--- save it ---");
        dumpDOM(document, new FileOutputStream("demo-dom.xml"));

        System.out.println("--- now read the newly created file ---");

        Document document2 = readAsDOM(new File("demo-dom.xml"));

        System.out.println("--- transform it to stdout ---");
        transformDOM(document2, new File("style.xsl"), System.out);

        System.out.println("--- transform directly file to file ---");
        transformFileToFile(new File("demo-dom.xml"), new File("style.xsl"),
                new File("demo-dom-out.xml"));

        System.out.println("--- precompile style first ---");
        Templates style = precompileStyle(new File("style.xsl"));
        
        System.out.println("--- and now transform again ---");
        transformFileToFileStylePrecompiled(new File("demo-dom.xml"), style,
                new File("demo-dom-out2.xml"));
    }

}
